from MetaConf import *
import tempfile
import os, pwd
import shutil
import re

def run(stack, scope, element, opts, parms):
    logger.debug("starting 'metafile' function")
    logger.debug("set='%s', comment='%s'" %
                 (element.get("set", ""),
                  element.get("comment", "#")))
    sets = stack[-1].get("metafilesets", [])[:]
    sets.extend(element.get("set", "").split())
    logger.debug("using sets: "+" ".join(sets))
    _filenames = element.text.split()
    if not _filenames:
        return
    filenames = []
    for filename in _filenames:
        if filename[:3] == "~*/":
            if os.getuid() == 0:
                filename = filename[2:]
                for home in [x[5] for x in pwd.getpwall()]:
                    path = home+filename
                    if os.path.isfile(path):
                        filenames.append(path)
        else:
            if filename[0] == "~":
                filename = os.path.expanduser(filename)
            if not os.path.isfile(filename):
                logger.warning("metafile not found: "+filename)
            else:
                filenames.append(filename)
    logger.debug("files to process: "+" ".join(filenames))
    comment = element.get("comment", "#")
    metafile = MetaFile(sets, parms, comment, opts.dryrun)
    for filename in filenames:
        metafile.process(filename)
    scope.error |= metafile.error
    logger.debug("finishing 'metafile' function")

# TODO: Remove this migration code.
OLDERROR = False
def checkold(comment, line):
    global OLDERROR
    if not OLDERROR:
        if line.startswith(comment+"|----"):
            logger.error("you're using the old format of metafiles, "
                         "check the documentation")
            OLDERROR = True

class MetaFile:
    def __init__(self, sets, parms, comment, dryrun=False):
        self.sets = sets
        self.parms = parms
        self.comment = comment
        self.dryrun = dryrun
        self.set = "%s:set:" % comment
        self.dis = "%s:disabled:" % comment
        self.inc = "%s:include:" % comment
        self.prm = "%s:parm:" % comment
        self.lset = len(self.set)
        self.lopt = len(self.dis)
        self.linc = len(self.inc)
        self.lprm = len(self.prm)
        self.error = False

    def process(self, filename):
        logger.info("processing metafile '%s'" % filename)
        logger.debug("reading original file")
        file = open(filename)
        lines = file.readlines()
        file.close()
        if self.dryrun:
            logger.debug("opening temporary file to emulate writting")
            file = tempfile.TemporaryFile()
        else:
            logger.debug("trying to create backup with a ~ suffix")
            try:
                os.unlink(filename+"~")
            except OSError:
                pass
            try:
                shutil.copy(filename, filename+"~")
            except OSError, e:
                logger.debug("failed to create backup: "+str(e))
                pass
            logger.debug("opening file for writting")
            file = open(filename, "w")
        logger.debug("starting to write new file")
        l = len(lines)
        i = 0
        # Uncommenting everything until the first set.
        i = self._uncomment(filename, file, lines, i, l)
        while i < l:
            if not lines[i].startswith(self.set):
                # This is not a set, and not part of a valid set.
                line = lines[i].lstrip()
                if line and not line.startswith(self.comment):
                    # Comment it out.
                    file.write(self.dis+lines[i])
                else:
                    # This is an empty line, or already commented.
                    file.write(lines[i])
                i += 1
            else:
                # This is a set.
                file.write(lines[i])
                evalstring = lines[i][self.lset:].strip()
                i += 1
                if not evalstring or evaluate(self.sets, evalstring):
                    i = self._uncomment(filename, file, lines, i, l)
        logger.debug("finished writting new file")

    def _uncomment(self, filename, file, lines, i, l):
        while i < l and not lines[i].startswith(self.set):
            # TODO: Remove this migration code.
            checkold(self.comment, lines[i])
            if lines[i].startswith(self.dis):
                file.write(lines[i][self.lopt:])
            elif lines[i].startswith(self.inc):
                file.write(lines[i])
                includei = i
                i += 1
                while i < l and not lines[i].startswith(self.inc):
                    i += 1
                if i == l:
                    # Save lines to avoid corrupting a wrong file
                    logger.error("metafile '%s' has unclosed include"
                                 % filename)
                    self.error = True
                    i = includei+1
                    while i < l:
                        file.write(lines[i])
                else:
                    incname = lines[includei][self.linc:].strip()
                    try:
                        incdata = open(incname).read()
                    except OSError:
                        logger.error("failed to include '%s' in "
                                     "metafile '%s'" % (incname, filename))
                        self.error = True
                    else:
                        file.write(incdata)
                    if not lines[i][self.linc:].strip():
                        # Include closure is empty. Write it and jump.
                        file.write(lines[i])
                        i += 1
                # Do not jump to next line
                continue
            else:
                file.write(lines[i])
                if lines[i].startswith(self.prm):
                    try:
                        file.write(lines[i][self.lprm:] % self.parms)
                    except (KeyError, ValueError), e:
                        if isinstance(e, KeyError):
                            logger.error("metafile '%s' requires undeclared "
                                         "parameter %s"
                                         % (filename, str(e)))
                        else:
                            logger.error("invalid parameter in metafile '%s'"
                                         % filename)
                        self.error = True
                        # We can't give up here, so write a commented line.
                        file.write(self.comment)
                        file.write(lines[i][self.lprm:])
                    i += 1
            i += 1
        return i

SPLIT = re.compile("\s*(\(|\)|&&|\|\||\s|!)\s*")

def evaluate(vars, expr):
    logger.debug("evaluating expression '%s'" % expr)
    toks = SPLIT.split(expr)
    res = _evaluate(vars, toks, 0, len(toks))[0]
    logger.debug("expression evaluated to %s" % `bool(res)`)
    return res

def _evaluate(vars, toks, i, l):
    anding = False
    inv = False
    res = None
    while i < l:
        tok = toks[i]
        if not tok:
            i += 1
            continue
        elif tok == '!':
            inv = True
            i += 1
            continue
        elif tok == '&&':
            anding = True
            i += 1
            continue
        elif tok in ')':
            return res, i
        elif tok == '(':
            if anding is False:
                if res is True:
                    break
                res = None
            if res is not False:
                res, i = _evaluate(vars, toks, i+1, l)
        elif tok == '||':
            if res is True:
                break
            res = None
        else:
            if anding is False:
                if res is True:
                    break
                res = None
            if res is not False:
                res = tok in vars
                if inv:
                    res = not res
        anding = False
        inv = False
        i += 1
    return res, i+1


# vim:sw=4:ts=4
