#!/bin/bash
# vi:nowrap
#
# by Andre Ruiz <andre@ruiz.net>
# version 1.2, 26 Nov 2003
# Please, send patches if you change it.
#
# This version needs metaconf >= 1.0
#
# script to activate one of your metaconf profiles in a dialog fashion :)
# metaconf was written by Gustavo Niemeyer, thanks Gustavo!
# this script requires packages diload and Xdialog for console and X support
#
# Sugestion: put this script to be run from inittab as a :respawn: service
# on tty10, so you can access it really fast anytime you need.
# mc:2345:respawn:/path-to/metaconf-chooser.sh -c </dev/tty10 >&0 2>&1

die() {
	echo $*
	exit 1
}

usage() {
		cat << EOF

Use: $(basename $0) <options>

Where options can be:

         -t <n>    Enter auto mode, where it waits an answer
                   for <n> seconds and exits without a selection.
                   This will also prevent looping (implies -o)
         -c        Do not display the "Exit" button, only the
                   "OK" button.
         -o        Ask the profile only once and exit instead
                   of looping until user selects 'Exit' button.

Written by Andre Ruiz <andre@conectiva.com.br>

EOF
}

dialog_path="$(type -path -a 'dialog')"
Xdialog_path="$(type -path -a 'Xdialog')"
profiles="$(metaconf -l)"
cancelbutton="--cancel-label Exit"
timeout=""
once=""
quiet=""

while [ -n "$1" ]; do
	case "$1" in
	"-h"|"--help")
		usage
		exit
		;;
	"-q")
		quiet="yes"
		;;
	"-c")
		cancelbutton="--no-cancel"
		;;
	"-t")
		[ -z "$2" ] && die "ERROR: missing argument to -t, see help"
		# will not check if it's a number, not worth
		timeout="--timeout $2"
		shift
		once="yes"
		;;
	"-o")
		once="yes"
		;;
	*)
		die "Invalid option \"$1\", aborting..."
		;;
	esac
	shift
done

if [ -z "$DISPLAY" ]; then
	dialog="dialog"
	if [ -z "$dialog_path" ]; then
		die "You don't have 'dialog' installed!"
	fi
else
	dialog="Xdialog"
	if [ -z "$Xdialog_path" ]; then
		[ -z "$quiet" ] && echo "You don't have 'Xdialog' installed, trying 'dialog'..."
		dialog="dialog"
		if [ -z "$dialog_path" ]; then
			die "You don't have neither 'dialog' nor 'Xdialog' installed!"
		fi
	fi
fi

commandtorun="$dialog $cancelbutton $timeout --menu \"Metaconf Profile Chooser\" 20 29 13"

[ -z "$profiles" ] && die "Error! 'metaconf -l' list is empty!"

set $profiles

while [ "$1" != "" ]; do
	commandtorun="$commandtorun \"$1\" \"\""
	shift
done

while :; do
    profile="$(eval $commandtorun 2>&1)"
    
    # will exit for a timeout, which implies -o option non-explicitly
    [ -z "$profile" -o "${profile:1}" = "timeout" ] && break

    [ -z "$quiet" ] && echo "Running metaconf for profile $profile..." 
    metaconf $profile

    # exit after the first run if asked
    [ -n "$once" ] && break

    sleep 1
done

